package info.bukova.isspst.services.requirement;

import info.bukova.isspst.Constants;
import info.bukova.isspst.data.RequirementState;
import info.bukova.isspst.data.TripRequirement;
import info.bukova.isspst.data.Workgroup;
import info.bukova.isspst.services.LazyLoader;
import info.bukova.isspst.services.workgroups.WorkgroupService;

import java.util.Date;
import java.util.List;

import org.hibernate.LazyInitializationException;
import org.hibernate.Query;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PostFilter;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;

public class TripRequirementServiceImpl extends RequirementBaseServiceImpl<TripRequirement>
		implements TripRequirementService, RequirementBaseService<TripRequirement> {
	
	@Autowired
	private RequirementTypeService reqTypeService;
	@Autowired
	private WorkgroupService workgroupService;

	@Override
	protected TripRequirement createEntity() {
		TripRequirement tr = new TripRequirement();
		tr.setReqDate(new Date());
		tr.setType(reqTypeService.getTypeById(Constants.REQTYPE_BUSINESSTRIP));
		tr.setState(RequirementState.NEW);
		addWorkflow(tr);
		return tr;
	}

	@Override
	@Transactional
	@PreAuthorize("hasPermission(this, 'PERM_ADD')")
	public void add(TripRequirement entity) {
		Workgroup reqWorkgroup = null;
		for (Workgroup w : workgroupService.getUserWorkgroups(getLoggedInUser())) {
			if (workgroupService.getMembers(entity.getCentre()).contains(w)) {
				reqWorkgroup = w;
			}
		}
		entity.setWorkgroup(reqWorkgroup);
		entity.setNumser(getNumberSerie());
		super.add(entity);
	}

	@SuppressWarnings("unchecked")
	@Override
	@Transactional
	@PreAuthorize("hasPermission(this, 'PERM_READ')")
	public List<TripRequirement> getAll() {
		Query q = dao.getQuery("from TripRequirement where ownedBy = :owner and state != :state");
		q.setParameter("owner", getLoggedInUser());
		q.setParameter("state", RequirementState.APPROVED);
		return q.list();
	}
	
	@SuppressWarnings("unchecked")
	@Override
	@Transactional
	@PreAuthorize("hasPermission(this, 'PERM_SHOW_CENTRE_REQ')")
	@PostFilter("hasPermission(filterObject, 'PERM_SHOW_CENTRE_REQ')")
	public List<TripRequirement> getCentreReq() {
		List<Workgroup> wgList = workgroupService.getUserCentres(getLoggedInUser());
		Query q = dao.getQuery("select tr from TripRequirement tr join fetch tr.ownedBy join tr.centre c where tr.state != :state and c in (:wgList) order by tr.numser");
		q.setParameterList("wgList", wgList);
		q.setParameter("state", RequirementState.APPROVED);
		return q.list();
	}

	@SuppressWarnings("unchecked")
	@Override
	@Transactional
	@PreAuthorize("hasPermission(this, 'PERM_SHOW_WORKGROUP_REQ')")
	@PostFilter("hasPermission(filterObject, 'PERM_SHOW_WORKGROUP_REQ')")
	public List<TripRequirement> getWorkgroupReq() {
		List<Workgroup> wgList = workgroupService.getUserWorkgroups(getLoggedInUser());
		Query q = dao.getQuery("select tr from TripRequirement tr join fetch tr.ownedBy join tr.workgroup w where tr.state != :state and w in (:wgList) order by tr.numser");
		q.setParameterList("wgList", wgList);
		q.setParameter("state", RequirementState.APPROVED);
		return q.list();
	}

	@SuppressWarnings("unchecked")
	@Override
	@Transactional
	@PreAuthorize("hasPermission(this, 'PERM_SHOW_ALL_REQ')")
	public List<TripRequirement> getFromAll() {
		Query q = dao.getQuery("from TripRequirement as tr join fetch tr.ownedBy order by tr.numser");
		return q.list();
	}

	@Override
	@Transactional
	@LazyLoader("form")
	public void loadPassangers(TripRequirement entity) {
		try {
			if (entity.getPassengers() == null) {
				throw new LazyInitializationException("");
			}
			
			entity.getPassengers().size();
		} catch (LazyInitializationException ex) {
			TripRequirement e = dao.getById(entity.getId());
			e.getPassengers().size();
			entity.setPassengers(e.getPassengers());	
		}
	}

}
